﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.Windows.Browser;
using System.Windows.Interop;
using MediaPlayer.Common;

namespace MediaPlayer
{
    public partial class Page : Canvas
    {
        #region User variable

        RandTransForms randerTransForms = new RandTransForms();

        enum medioPlayState { play, stop, pause }
        medioPlayState playState = medioPlayState.stop;

        Downloader _downloader = new Downloader();

        private double MouseDragBeginX = 0;
        private bool MouseDragDown = false;

        #endregion

        #region "Page_Loaded"

        public void Page_Loaded(object o, EventArgs e)
        {
            // Required to initialize variables
            InitializeComponent();

            // 링크 이벤트
            OnLink.MouseLeftButtonDown += new MouseEventHandler(OnLink_MouseLeftButtonDown);

            // 플스크린 이벤트
            MultiMedia.MouseLeftButtonDown += new MouseEventHandler(MultiMedia_MouseLeftButtonDown);

            OnFullScreen.MouseEnter += new MouseEventHandler(OnFullScreen_MouseEnter);
            OnFullScreen.MouseLeave += new EventHandler(OnFullScreen_MouseLeave);
            OnFullScreen.MouseLeftButtonDown += new MouseEventHandler(OnFullScreen_MouseLeftButtonDown);

            BrowserHost.FullScreenChange += new EventHandler(BrowserHost_FullScreenChange);

            // 재생(Play)
            OnPlay.MouseEnter += new MouseEventHandler(OnPlay_MouseEnter);
            OnPlay.MouseLeave += new EventHandler(OnPlay_MouseLeave);
            OnPlay.MouseLeftButtonDown += new MouseEventHandler(OnPlay_MouseLeftButtonDown);

            // 일시정지(Pause)
            OnPause.MouseEnter += new MouseEventHandler(OnPause_MouseEnter);
            OnPause.MouseLeave += new EventHandler(OnPause_MouseLeave);
            OnPause.MouseLeftButtonDown += new MouseEventHandler(OnPause_MouseLeftButtonDown);

            // 정지(Stop)
            OnStop.MouseEnter += new MouseEventHandler(OnStop_MouseEnter);
            OnStop.MouseLeave += new EventHandler(OnStop_MouseLeave);
            OnStop.MouseLeftButtonDown += new MouseEventHandler(OnStop_MouseLeftButtonDown);

            //Media Download...
            string source = "Media/adoverlay.wmv";
            _downloader.Open("GET", new Uri(source, UriKind.Relative));
            _downloader.Send();

            // DownLoad Progress
            _downloader.DownloadProgressChanged += new EventHandler(_downloader_DownloadProgressChanged);
            _downloader.Completed += new EventHandler(_downloader_Completed);

            // Play Progress / Location
            PlayProgress.MouseLeftButtonDown += new MouseEventHandler(PlayProgress_MouseLeftButtonDown);
            DownProgress.MouseLeftButtonDown += new MouseEventHandler(DownProgress_MouseLeftButtonDown);
            PlayLocation.MouseLeftButtonDown += new MouseEventHandler(PlayLocation_MouseLeftButtonDown);
            PlayLocation.MouseMove += new MouseEventHandler(PlayLocation_MouseMove);
            PlayLocation.MouseLeftButtonUp += new MouseEventHandler(PlayLocation_MouseLeftButtonUp);

            // MediaElement
            timerPlayProgress.Completed += new EventHandler(timerPlayProgress_Completed);
            MultiMedia.CurrentStateChanged += new EventHandler(MultiMedia_CurrentStateChanged);

            // 볼륨
            MuteButton.MouseLeftButtonDown += new MouseEventHandler(MuteButton_MouseLeftButtonDown);
            MuteOnSymbol.Visibility = Visibility.Collapsed;
            VolumeDuration.MouseLeftButtonDown += new MouseEventHandler(VolumeDuration_MouseLeftButtonDown);
            VolumeProgress.MouseLeftButtonDown += new MouseEventHandler(VolumeProgress_MouseLeftButtonDown);

            // 재생시간
            MultiMedia.MediaOpened += new EventHandler(MultiMedia_MediaOpened);

            // Chapters 애니메이션
            CnvChapter.MouseEnter += new MouseEventHandler(CnvChapter_MouseEnter);
            CnvChapter.MouseLeave += new EventHandler(CnvChapter_MouseLeave);

            // Chapters
            Ch1.MouseLeftButtonDown += new MouseEventHandler(Ch1_MouseLeftButtonDown);
            Ch2.MouseLeftButtonDown += new MouseEventHandler(Ch2_MouseLeftButtonDown);
            Ch3.MouseLeftButtonDown += new MouseEventHandler(Ch3_MouseLeftButtonDown);
            Ch4.MouseLeftButtonDown += new MouseEventHandler(Ch4_MouseLeftButtonDown);
            Ch5.MouseLeftButtonDown += new MouseEventHandler(Ch5_MouseLeftButtonDown);
            Ch6.MouseLeftButtonDown += new MouseEventHandler(Ch6_MouseLeftButtonDown);
            Ch7.MouseLeftButtonDown += new MouseEventHandler(Ch7_MouseLeftButtonDown);
        }


        #endregion

        #region "Link"

        void OnLink_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            HtmlPage.Navigate("http://www.netfighter.pe.kr", "_blenk");
        }

        #endregion

        #region "Full Screen"

        void MultiMedia_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            BrowserHost.IsFullScreen = !BrowserHost.IsFullScreen;
            ResizeControl();
        }

        void OnFullScreen_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            BrowserHost.IsFullScreen = !BrowserHost.IsFullScreen;
            ResizeControl();
        }

        void OnFullScreen_MouseEnter(object sender, MouseEventArgs e)
        {
            OnFullScreen.RenderTransform = randerTransForms.scaleTransform(1.3, 1.3);
        }

        void OnFullScreen_MouseLeave(object sender, EventArgs e)
        {
            OnFullScreen.RenderTransform = randerTransForms.scaleTransform(1, 1);
        }

        void BrowserHost_FullScreenChange(object sender, EventArgs e)
        {
            ResizeControl();
        }

        void ResizeControl()
        {
            double scale = Math.Min(BrowserHost.ActualHeight / this.Height, BrowserHost.ActualWidth / this.Width);

            scanTransfrom.ScaleX = scale;
            scanTransfrom.ScaleY = scale;
        }

        #endregion

        #region "Play"

        private void OnPlay_MouseEnter(object sender, MouseEventArgs e)
        {
            OnPlay.RenderTransform = randerTransForms.scaleTransform(1.3, 1.3);
        }

        private void OnPlay_MouseLeave(object sender, EventArgs e)
        {
            OnPlay.RenderTransform = randerTransForms.scaleTransform(1, 1);
        }

        private void OnPlay_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            if (playState != medioPlayState.play)
            {
                MultiMedia.Play();
                playState = medioPlayState.play;
                OnPause.Visibility = Visibility.Visible;
                OnPlay.Visibility = Visibility.Collapsed;

                timerPlayProgress.Begin();
            }
        }

        #endregion

        #region "Pause"

        private void OnPause_MouseEnter(object sender, MouseEventArgs e)
        {
            OnPause.RenderTransform = randerTransForms.scaleTransform(1.3, 1.3);
        }

        private void OnPause_MouseLeave(object sender, EventArgs e)
        {
            OnPause.RenderTransform = randerTransForms.scaleTransform(1, 1);
        }

        private void OnPause_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            if (playState != medioPlayState.pause)
            {
                MultiMedia.Pause();
                playState = medioPlayState.pause;
                OnPause.Visibility = Visibility.Collapsed;
                OnPlay.Visibility = Visibility.Visible;

                timerPlayProgress.Pause();
            }
        }

        #endregion

        #region "Stop"

        private void OnStop_MouseEnter(object sender, MouseEventArgs e)
        {
            OnStop.RenderTransform = randerTransForms.scaleTransform(1.3, 1.3);
        }

        private void OnStop_MouseLeave(object sender, EventArgs e)
        {
            OnStop.RenderTransform = randerTransForms.scaleTransform(1, 1);
        }

        private void OnStop_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            if (playState != medioPlayState.stop)
            {
                MultiMedia.Stop();
                playState = medioPlayState.stop;
                OnPause.Visibility = Visibility.Collapsed;
                OnPlay.Visibility = Visibility.Visible;

                timerPlayProgress.Stop();

                txPlayCurrentTime.Text = "00:00:00";

                PlayLocation.SetValue(Canvas.LeftProperty, 
                                          PlayDuration.GetValue(Canvas.LeftProperty));

                PlayProgress.SetValue(Canvas.WidthProperty, 1.793);
            }
        }

        #endregion

        #region "Download Progress"

        private void _downloader_DownloadProgressChanged(object sender, EventArgs e)
        {
            var done = _downloader.DownloadProgress;

            //ProgressBar State...
            DownProgress.SetValue(Canvas.WidthProperty, done * 410);

            // Ex) 78%
            txtProgress.Text = Math.Floor(done * 100).ToString() + "%";
        }

        private void _downloader_Completed(object sender, EventArgs e)
        {
            // Media Source Loading
            MultiMedia.SetSource(_downloader, null);
        }

        #endregion

        #region "Play Progress / Location"

        private void PlayProgress_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            TimeSpan d = MultiMedia.NaturalDuration.TimeSpan;
            double duration = d.Hours * 3600 + d.Minutes * 60 + (double)d.Seconds;
            Point pt = e.GetPosition(PlayDuration);

            MultiMedia.Position = TimeSpan.FromSeconds(duration * (pt.X / PlayDuration.Width));
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void DownProgress_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            TimeSpan d = MultiMedia.NaturalDuration.TimeSpan;
            double duration = d.Hours * 3600 + d.Minutes * 60 + (double)d.Seconds;
            Point pt = e.GetPosition(PlayDuration);

            MultiMedia.Position = TimeSpan.FromSeconds(duration * (pt.X / PlayDuration.Width));
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void PlayLocation_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MouseDragBeginX = e.GetPosition(PlayDuration).X;
            MouseDragDown = true;
            PlayLocation.CaptureMouse();
            OnPause_MouseLeftButtonDown(sender, null);
        }

        private void PlayLocation_MouseMove(object sender, MouseEventArgs e)
        {
            if (MouseDragDown)
            {
                double currX = e.GetPosition(PlayDuration).X;

                double xValue = 
                    double.Parse(PlayLocation.GetValue(Canvas.LeftProperty).ToString());

                xValue += (currX - MouseDragBeginX);

                double moveAreaMinX = Convert.ToDouble(PlayDuration.GetValue(Canvas.LeftProperty)) - (PlayLocation.Width / 2);

                double moveAreaMaxX = moveAreaMinX + PlayDuration.Width;

                if (!(xValue < moveAreaMinX || xValue > moveAreaMaxX))
                {
                    PlayLocation.SetValue<double>(Canvas.LeftProperty, xValue);
                    MouseDragBeginX = currX;
                }
            }
        }

        private void PlayLocation_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            if (MouseDragDown)
            {
                MouseDragDown = false;
                PlayLocation.ReleaseMouseCapture();

                if (e != null)
                    DownProgress_MouseLeftButtonDown(sender, e);
            }
        }

        #endregion

        #region MediaElement

        // Media Play State Progress Stroyboard 
        private void timerPlayProgress_Completed(object sender, EventArgs e)
        {
           TimeSpan position = MultiMedia.Position;

            if (position != null)
            {
                double width = position.Hours * 3600 
                             + position.Minutes * 60 
                             + position.Seconds;

                txPlayCurrentTime.Text = string.Format("{0:D2}:{1:D2}:{2:D2}", 
                                         position.Hours, position.Minutes, position.Seconds);
             
                TimeSpan d = MultiMedia.NaturalDuration.TimeSpan;
                double duration = d.Hours * 3600 + d.Minutes * 60 + d.Seconds;
                width = width * Convert.ToDouble(PlayDuration.Width) / duration;

                PlayLocation.SetValue(Canvas.LeftProperty, width 
                             + Convert.ToDouble(PlayDuration.GetValue(Canvas.LeftProperty))
                             - (PlayLocation.Width / 2));

                PlayProgress.SetValue(Canvas.WidthProperty, width);

                Console.WriteLine(MultiMedia.DownloadProgress);

                if (MultiMedia.CurrentState == "Paused")
                {
                    MultiMedia.Stop();
                    OnPlay.Visibility = Visibility.Visible;
                    OnPause.Visibility = Visibility.Collapsed;

                    OnStop_MouseLeftButtonDown(sender, null);
                }
            }

            timerPlayProgress.Begin();
        }

        private void MultiMedia_CurrentStateChanged(object sender, EventArgs e)
        {
            textCurrent.Text = MultiMedia.CurrentState;
        }

        #endregion

        #region "Volume"

        private void MuteButton_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.IsMuted = !MultiMedia.IsMuted;

            SolidColorBrush brush = new SolidColorBrush();
            if (MultiMedia.IsMuted)
            {
                Path_17.Visibility = Visibility.Visible;
                MuteOnSymbol.Visibility = Visibility.Visible;
                MuteOffSymbol.Visibility = Visibility.Visible;
            }
            else
            {
                Path_17.Visibility = Visibility.Visible;
                MuteOnSymbol.Visibility = Visibility.Collapsed;
                MuteOffSymbol.Visibility = Visibility.Visible;
            }
        }

        private void VolumeDuration_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            double pt = e.GetPosition(VolumeDuration).X;
            VolumeProgress.SetValue(Canvas.WidthProperty, pt);
            MultiMedia.Volume = Math.Round(pt / 152, 1);
        }

        private void VolumeProgress_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            double pt = e.GetPosition(VolumeDuration).X;
            VolumeProgress.SetValue(Canvas.WidthProperty, pt);
            MultiMedia.Volume = Math.Round(pt / 152, 1);
        }

        #endregion 

        #region "재생시간"

        private void MultiMedia_MediaOpened(object sender, EventArgs e)
        {
            // Media Play Time
            txPlayTotalTime.Text = string.Format("{0:D2}:{1:D2}:{2:D2}"
                                                 , MultiMedia.NaturalDuration.TimeSpan.Hours
                                                 , MultiMedia.NaturalDuration.TimeSpan.Minutes
                                                 , MultiMedia.NaturalDuration.TimeSpan.Seconds);
        }

        #endregion

        #region "hapters 애니메이션"

        void CnvChapter_MouseEnter(object sender, MouseEventArgs e)
        {
             ChaptersShowAni.Begin();
            ChaptersHiddenAni.Stop();
        }

        void CnvChapter_MouseLeave(object sender, EventArgs e)
        {
            ChaptersHiddenAni.Begin();
            ChaptersShowAni.Stop();
        }

        #endregion

        #region "Chapters"

        private void Ch1_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(60); // Position Time : 01:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch2_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {

            MultiMedia.Position = TimeSpan.FromSeconds(120); // Position Time : 02:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch3_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(180); // Position Time : 03:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch4_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(240); // Position Time : 04:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch5_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(300); // Position Time : 05:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch6_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(360); // Position Time : 06:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        private void Ch7_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            MultiMedia.Position = TimeSpan.FromSeconds(420); // Position Time : 07:00
            OnPlay_MouseLeftButtonDown(sender, null);
        }

        #endregion

    }
}
